@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 12/6/2019.
-//  Copyright (c) 2019-2025 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2019-2025 Benjamin C. Haller.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -446,7 +446,11 @@ QString QtSLiMEidosConsole::_executeScriptString(QString scriptString, QString *
     parentSLiMWindow->willExecuteScript();
 	
     std::ostringstream outstream;	// in the Eidos console, one output stream for both types of output
-	EidosInterpreter interpreter(script, *global_symbols, *global_function_map, eidos_context, outstream, outstream);
+	EidosInterpreter interpreter(script, *global_symbols, *global_function_map, eidos_context, outstream, outstream
+#ifdef SLIMGUI
+            , true
+#endif
+            );
 	
 	try
 	{
