@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 1/19/2024.
-//  Copyright (c) 2024-2025 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2024-2025 Benjamin C. Haller.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -73,6 +73,9 @@ void QtSLiMGraphView_CustomPlot::freeData(void)
     for (std::vector<QColor> *borderbuffer : border_)
         delete borderbuffer;
     
+    for (std::vector<double> *alphabuffer : alpha_)
+        delete alphabuffer;
+    
     for (std::vector<double> *lwdbuffer : line_width_)
         delete lwdbuffer;
     
@@ -87,10 +90,12 @@ void QtSLiMGraphView_CustomPlot::freeData(void)
     symbol_.clear();
     color_.clear();
     border_.clear();
+    alpha_.clear();
     line_width_.clear();
     size_.clear();
     xadj_.clear();
     yadj_.clear();
+    image_.clear();
     
     // reset the legend state
     legend_added_ = false;
@@ -349,7 +354,8 @@ void QtSLiMGraphView_CustomPlot::rescaleAxesForDataRange(void)
 
 void QtSLiMGraphView_CustomPlot::addABLineData(double *a_values, double *b_values,
                                                double *h_values, double *v_values, int data_count,
-                                               std::vector<QColor> *color, std::vector<double> *lwd)
+                                               std::vector<QColor> *color, std::vector<double> *alpha,
+                                               std::vector<double> *lwd)
 {
     if (a_values)
     {
@@ -375,17 +381,20 @@ void QtSLiMGraphView_CustomPlot::addABLineData(double *a_values, double *b_value
     symbol_.push_back(nullptr);             // unused for abline
     color_.push_back(color);
     border_.push_back(nullptr);             // unused for abline
+    alpha_.push_back(alpha);
     line_width_.push_back(lwd);
     size_.push_back(nullptr);               // unused for abline
     xadj_.push_back(-1);                    // unused for abline
     yadj_.push_back(-1);                    // unused for abline
+    image_.push_back(QImage());             // unused for abline
     
     //rescaleAxesForDataRange();            // not needed for abline
     update();
 }
 
 void QtSLiMGraphView_CustomPlot::addLineData(double *x_values, double *y_values, int data_count,
-                                             std::vector<QColor> *color, std::vector<double> *lwd)
+                                             std::vector<QColor> *color, std::vector<double> *alpha,
+                                             std::vector<double> *lwd)
 {
     plot_type_.push_back(QtSLiM_CustomPlotType::kLines);
     xdata_.push_back(x_values);
@@ -395,10 +404,12 @@ void QtSLiMGraphView_CustomPlot::addLineData(double *x_values, double *y_values,
     symbol_.push_back(nullptr);             // unused for lines
     color_.push_back(color);
     border_.push_back(nullptr);             // unused for lines
+    alpha_.push_back(alpha);
     line_width_.push_back(lwd);
     size_.push_back(nullptr);               // unused for lines
     xadj_.push_back(-1);                    // unused for lines
     yadj_.push_back(-1);                    // unused for lines
+    image_.push_back(QImage());             // unused for lines
     
     rescaleAxesForDataRange();
     update();
@@ -406,7 +417,7 @@ void QtSLiMGraphView_CustomPlot::addLineData(double *x_values, double *y_values,
 
 void QtSLiMGraphView_CustomPlot::addPointData(double *x_values, double *y_values, int data_count,
                                               std::vector<int> *symbol, std::vector<QColor> *color, std::vector<QColor> *border,
-                                              std::vector<double> *lwd, std::vector<double> *size)
+                                              std::vector<double> *alpha, std::vector<double> *lwd, std::vector<double> *size)
 {
     plot_type_.push_back(QtSLiM_CustomPlotType::kPoints);
     xdata_.push_back(x_values);
@@ -416,17 +427,20 @@ void QtSLiMGraphView_CustomPlot::addPointData(double *x_values, double *y_values
     symbol_.push_back(symbol);
     color_.push_back(color);
     border_.push_back(border);
+    alpha_.push_back(alpha);
     line_width_.push_back(lwd);
     size_.push_back(size);
     xadj_.push_back(-1);                    // unused for points
     yadj_.push_back(-1);                    // unused for points
+    image_.push_back(QImage());             // unused for points
     
     rescaleAxesForDataRange();
     update();
 }
 
 void QtSLiMGraphView_CustomPlot::addTextData(double *x_values, double *y_values, std::vector<QString> *labels, int data_count,
-                                             std::vector<QColor> *color, std::vector<double> *size, double *adj)
+                                             std::vector<QColor> *color, std::vector<double> *alpha,
+                                             std::vector<double> *size, double *adj)
 {
     plot_type_.push_back(QtSLiM_CustomPlotType::kText);
     xdata_.push_back(x_values);
@@ -436,10 +450,34 @@ void QtSLiMGraphView_CustomPlot::addTextData(double *x_values, double *y_values,
     symbol_.push_back(nullptr);             // unused for text
     color_.push_back(color);
     border_.push_back(nullptr);             // unused for text
+    alpha_.push_back(alpha);
     line_width_.push_back(nullptr);         // unused for text
     size_.push_back(size);
     xadj_.push_back(adj[0]);
     yadj_.push_back(adj[1]);
+    image_.push_back(QImage());             // unused for text
+    
+    rescaleAxesForDataRange();
+    update();
+}
+
+void QtSLiMGraphView_CustomPlot::addImageData(double *x_values, double *y_values, int data_count,
+                  QImage image, std::vector<double> *alpha)
+{
+    plot_type_.push_back(QtSLiM_CustomPlotType::kImage);
+    xdata_.push_back(x_values);
+    ydata_.push_back(y_values);
+    labels_.push_back(nullptr);             // unused for image
+    data_count_.push_back(data_count);
+    symbol_.push_back(nullptr);             // unused for image
+    color_.push_back(nullptr);              // unused for image
+    border_.push_back(nullptr);             // unused for image
+    alpha_.push_back(alpha);
+    line_width_.push_back(nullptr);         // unused for image
+    size_.push_back(nullptr);               // unused for image
+    xadj_.push_back(-1);                    // unused for image
+    yadj_.push_back(-1);                    // unused for image
+    image_.push_back(image);
     
     rescaleAxesForDataRange();
     update();
@@ -478,6 +516,12 @@ void QtSLiMGraphView_CustomPlot::addLegendSwatchEntry(QString label, QColor colo
     update();
 }
 
+void QtSLiMGraphView_CustomPlot::addLegendTitleEntry(QString label)
+{
+    legend_entries_.emplace_back(label);
+    update();
+}
+
 QString QtSLiMGraphView_CustomPlot::graphTitle(void)
 {
     return title_;
@@ -515,6 +559,9 @@ void QtSLiMGraphView_CustomPlot::drawGraph(QPainter &painter, QRect interiorRect
         case QtSLiM_CustomPlotType::kVLines:
             drawVLines(painter, interiorRect, i);
             break;
+        case QtSLiM_CustomPlotType::kImage:
+            drawImage(painter, interiorRect, i);
+            break;
         }
     }
 }
@@ -550,8 +597,9 @@ void QtSLiMGraphView_CustomPlot::drawABLines(QPainter &painter, QRect interiorRe
     double *adata = xdata_[dataIndex];
     double *bdata = ydata_[dataIndex];
     int lineCount = data_count_[dataIndex];
-    std::vector<QColor> &lineColors = *color_[dataIndex];           // might be one value of N values
-    std::vector<double> &lineWidths = *line_width_[dataIndex];      // might be one value of N values
+    std::vector<QColor> &lineColors = *color_[dataIndex];           // might be one value or N values
+    std::vector<double> &lineAlphas = *alpha_[dataIndex];           // might be one value or N values
+    std::vector<double> &lineWidths = *line_width_[dataIndex];      // might be one value or N values
     
     for (int lineIndex = 0; lineIndex < lineCount; ++lineIndex)
     {
@@ -569,11 +617,15 @@ void QtSLiMGraphView_CustomPlot::drawABLines(QPainter &painter, QRect interiorRe
             QPointF devicePoint1(plotToDeviceX(user_x1, interiorRect), plotToDeviceY(user_y1, interiorRect));
             QPointF devicePoint2(plotToDeviceX(user_x2, interiorRect), plotToDeviceY(user_y2, interiorRect));
             QColor lineColor = lineColors[lineIndex % lineColors.size()];
+            double lineAlpha = lineAlphas[lineIndex % lineAlphas.size()];
             double lineWidth = lineWidths[lineIndex % lineWidths.size()];
             
             linePath.moveTo(devicePoint1);
             linePath.lineTo(devicePoint2);
             
+            if (lineAlpha != 1.0)
+                lineColor.setAlphaF(lineAlpha);
+            
             painter.strokePath(linePath, QPen(lineColor, lineWidth));
         }
     }
@@ -583,8 +635,9 @@ void QtSLiMGraphView_CustomPlot::drawHLines(QPainter &painter, QRect interiorRec
 {
     double *hdata = xdata_[dataIndex];
     int lineCount = data_count_[dataIndex];
-    std::vector<QColor> &lineColors = *color_[dataIndex];           // might be one value of N values
-    std::vector<double> &lineWidths = *line_width_[dataIndex];      // might be one value of N values
+    std::vector<QColor> &lineColors = *color_[dataIndex];           // might be one value or N values
+    std::vector<double> &lineAlphas = *alpha_[dataIndex];           // might be one value or N values
+    std::vector<double> &lineWidths = *line_width_[dataIndex];      // might be one value or N values
     
     for (int lineIndex = 0; lineIndex < lineCount; ++lineIndex)
     {
@@ -597,11 +650,15 @@ void QtSLiMGraphView_CustomPlot::drawHLines(QPainter &painter, QRect interiorRec
             QPointF devicePoint1(plotToDeviceX(x0_ - 100000.0, interiorRect), roundPlotToDeviceY(user_h, interiorRect));
             QPointF devicePoint2(plotToDeviceX(x1_ + 100000.0, interiorRect), roundPlotToDeviceY(user_h, interiorRect));
             QColor lineColor = lineColors[lineIndex % lineColors.size()];
+            double lineAlpha = lineAlphas[lineIndex % lineAlphas.size()];
             double lineWidth = lineWidths[lineIndex % lineWidths.size()];
             
             linePath.moveTo(devicePoint1);
             linePath.lineTo(devicePoint2);
             
+            if (lineAlpha != 1.0)
+                lineColor.setAlphaF(lineAlpha);
+            
             painter.strokePath(linePath, QPen(lineColor, lineWidth));
         }
     }
@@ -611,8 +668,9 @@ void QtSLiMGraphView_CustomPlot::drawVLines(QPainter &painter, QRect interiorRec
 {
     double *vdata = xdata_[dataIndex];
     int lineCount = data_count_[dataIndex];
-    std::vector<QColor> &lineColors = *color_[dataIndex];           // might be one value of N values
-    std::vector<double> &lineWidths = *line_width_[dataIndex];      // might be one value of N values
+    std::vector<QColor> &lineColors = *color_[dataIndex];           // might be one value or N values
+    std::vector<double> &lineAlphas = *alpha_[dataIndex];           // might be one value or N values
+    std::vector<double> &lineWidths = *line_width_[dataIndex];      // might be one value or N values
     
     for (int lineIndex = 0; lineIndex < lineCount; ++lineIndex)
     {
@@ -625,11 +683,15 @@ void QtSLiMGraphView_CustomPlot::drawVLines(QPainter &painter, QRect interiorRec
             QPointF devicePoint1(roundPlotToDeviceX(user_v, interiorRect), plotToDeviceY(y0_ - 100000.0, interiorRect));
             QPointF devicePoint2(roundPlotToDeviceX(user_v, interiorRect), plotToDeviceY(y1_ + 100000.0, interiorRect));
             QColor lineColor = lineColors[lineIndex % lineColors.size()];
+            double lineAlpha = lineAlphas[lineIndex % lineAlphas.size()];
             double lineWidth = lineWidths[lineIndex % lineWidths.size()];
             
             linePath.moveTo(devicePoint1);
             linePath.lineTo(devicePoint2);
             
+            if (lineAlpha != 1.0)
+                lineColor.setAlphaF(lineAlpha);
+            
             painter.strokePath(linePath, QPen(lineColor, lineWidth));
         }
     }
@@ -640,34 +702,68 @@ void QtSLiMGraphView_CustomPlot::drawLines(QPainter &painter, QRect interiorRect
     double *xdata = xdata_[dataIndex];
     double *ydata = ydata_[dataIndex];
     int pointCount = data_count_[dataIndex];
-    QColor lineColor = (*color_[dataIndex])[0];         // guaranteed to be only one value, for plotLines()
-    double lineWidth = (*line_width_[dataIndex])[0];    // guaranteed to be only one value, for plotLines()
+    QColor lineColor = (*color_[dataIndex])[0];                 // guaranteed to be only one value, for plotLines()
+    double lineAlpha = (*alpha_[dataIndex])[0];                 // guaranteed to be only one value, for plotLines()
+    double lineWidth = (*line_width_[dataIndex])[0];            // guaranteed to be only one value, for plotLines()
     
-    QPainterPath linePath;
-    bool startedLine = false;
-    
-    for (int pointIndex = 0; pointIndex < pointCount; ++pointIndex)
+    if (lineAlpha != 1.0)
     {
-        double user_x = xdata[pointIndex];
-        double user_y = ydata[pointIndex];
+        // This strokes each line segment as a separate path, so that when successive line segments cross, the alpha
+        // value affects their area of overlap.  This is arguably more likely to be what the user expects.  However,
+        // it doesn't draw the line joins nicely, with bevels and such, so the line path as a whole might less pretty.
+        // We therefore use this drawing method only when alpha is not 1.0.
+        lineColor.setAlphaF(lineAlpha);
         
-        if (!std::isnan(user_x) && !std::isnan(user_y))
+        for (int lineIndex = 0; lineIndex < pointCount - 1; ++lineIndex)
         {
-            QPointF devicePoint(plotToDeviceX(user_x, interiorRect), plotToDeviceY(user_y, interiorRect));
+            QPainterPath linePath;
+            double user_x1 = xdata[lineIndex];
+            double user_y1 = ydata[lineIndex];
+            double user_x2 = xdata[lineIndex + 1];
+            double user_y2 = ydata[lineIndex + 1];
             
-            if (startedLine)    linePath.lineTo(devicePoint);
-            else                linePath.moveTo(devicePoint);
-            
-            startedLine = true;
-        }
-        else
-        {
-            // a NAN value for x or y interrupts the line being plotted; INF values are plotted, but don't affect axis ranges
-            startedLine = false;
+            if (!std::isnan(user_x1) && !std::isnan(user_y1) && !std::isnan(user_x2) && !std::isnan(user_y2))
+            {
+                QPointF devicePoint1(plotToDeviceX(user_x1, interiorRect), plotToDeviceY(user_y1, interiorRect));
+                QPointF devicePoint2(plotToDeviceX(user_x2, interiorRect), plotToDeviceY(user_y2, interiorRect));
+                
+                linePath.moveTo(devicePoint1);
+                linePath.lineTo(devicePoint2);
+                painter.strokePath(linePath, QPen(lineColor, lineWidth));
+            }
         }
     }
-    
-    painter.strokePath(linePath, QPen(lineColor, lineWidth));
+    else
+    {
+        QPainterPath linePath;
+        bool startedLine = false;
+        
+        for (int lineIndex = 0; lineIndex < pointCount; ++lineIndex)
+        {
+            double user_x = xdata[lineIndex];
+            double user_y = ydata[lineIndex];
+            
+            if (!std::isnan(user_x) && !std::isnan(user_y))
+            {
+                QPointF devicePoint(plotToDeviceX(user_x, interiorRect), plotToDeviceY(user_y, interiorRect));
+                
+                if (startedLine)    linePath.lineTo(devicePoint);
+                else                linePath.moveTo(devicePoint);
+                
+                startedLine = true;
+            }
+            else
+            {
+                // a NAN value for x or y interrupts the line being plotted; INF values are plotted, but don't affect axis ranges
+                startedLine = false;
+            }
+        }
+        
+        if (lineAlpha != 1.0)
+            lineColor.setAlphaF(lineAlpha);
+        
+        painter.strokePath(linePath, QPen(lineColor, lineWidth));
+    }
 }
 
 void QtSLiMGraphView_CustomPlot::drawPoints(QPainter &painter, QRect interiorRect, int dataIndex)
@@ -678,6 +774,7 @@ void QtSLiMGraphView_CustomPlot::drawPoints(QPainter &painter, QRect interiorRec
     std::vector<int> &symbols = *symbol_[dataIndex];
     std::vector<QColor> &symbolColors = *color_[dataIndex];
     std::vector<QColor> &borderColors = *border_[dataIndex];
+    std::vector<double> &alphas = *alpha_[dataIndex];
     std::vector<double> &lineWidths = *line_width_[dataIndex];
     std::vector<double> &sizes = *size_[dataIndex];
     
@@ -688,6 +785,7 @@ void QtSLiMGraphView_CustomPlot::drawPoints(QPainter &painter, QRect interiorRec
         int symbol = symbols[pointIndex % symbols.size()];
         QColor symbolColor = symbolColors[pointIndex % symbolColors.size()];
         QColor borderColor = borderColors[pointIndex % borderColors.size()];
+        double alpha = alphas[pointIndex % alphas.size()];
         double lineWidth = lineWidths[pointIndex % lineWidths.size()];
         double size = sizes[pointIndex % sizes.size()];
         
@@ -697,7 +795,7 @@ void QtSLiMGraphView_CustomPlot::drawPoints(QPainter &painter, QRect interiorRec
             double x = plotToDeviceX(user_x, interiorRect);
             double y = plotToDeviceY(user_y, interiorRect);
             
-            drawPointSymbol(painter, x, y, symbol, symbolColor, borderColor, lineWidth, size);
+            drawPointSymbol(painter, x, y, symbol, symbolColor, borderColor, alpha, lineWidth, size);
         }
     }
 }
@@ -709,6 +807,7 @@ void QtSLiMGraphView_CustomPlot::drawText(QPainter &painter, QRect interiorRect,
     std::vector<QString> &labels = *labels_[dataIndex];
     int pointCount = data_count_[dataIndex];
     std::vector<QColor> &textColors = *color_[dataIndex];
+    std::vector<double> &textAlphas = *alpha_[dataIndex];
     std::vector<double> &pointSizes = *size_[dataIndex];
     double xadj = xadj_[dataIndex];
     double yadj = yadj_[dataIndex];
@@ -739,6 +838,10 @@ void QtSLiMGraphView_CustomPlot::drawText(QPainter &painter, QRect interiorRect,
             }
             
             QColor textColor = textColors[pointIndex % textColors.size()];
+            double alpha = textAlphas[pointIndex % textAlphas.size()];
+            
+            if (alpha != 1.0)
+                textColor.setAlphaF(alpha);
             
             painter.setPen(textColor);
             
@@ -777,6 +880,39 @@ void QtSLiMGraphView_CustomPlot::drawText(QPainter &painter, QRect interiorRect,
     }
 }
 
+void QtSLiMGraphView_CustomPlot::drawImage(QPainter &painter, QRect interiorRect, int dataIndex)
+{
+    double *xdata = xdata_[dataIndex];
+    double *ydata = ydata_[dataIndex];
+    std::vector<double> &imageAlphas = *alpha_[dataIndex];
+    double alpha = imageAlphas[0];
+    
+    double user_x1 = xdata[0], user_y1 = ydata[0];
+    double user_x2 = xdata[1], user_y2 = ydata[1];
+    
+    double x1 = plotToDeviceX(user_x1, interiorRect);
+    double y1 = plotToDeviceY(user_y1, interiorRect);
+    double x2 = plotToDeviceX(user_x2, interiorRect);
+    double y2 = plotToDeviceY(user_y2, interiorRect);
+    
+    // the coordinates are absolute, but Qt wants them as width/height
+    double target_width = x2 - x1;
+    double target_height = y2 - y1;
+    
+    // get the image data
+    const QImage &image = image_[dataIndex];
+    
+    QRectF target(x1, y1, target_width, target_height);
+    
+    if (alpha != 1.0)
+        painter.setOpacity(alpha);
+    
+    painter.drawImage(target, image);
+    
+    if (alpha != 1.0)
+        painter.setOpacity(1.0);
+}
+
 
 
 
