@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 1/19/2024.
-//  Copyright (c) 2024-2025 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2024-2025 Benjamin C. Haller.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -36,6 +36,7 @@ enum class QtSLiM_CustomPlotType : int {
     kABLines,   // from abline()
     kHLines,    // from abline()
     kVLines,    // from abline()
+    kImage,
 };
 
 class QtSLiMGraphView_CustomPlot : public QtSLiMGraphView
@@ -65,20 +66,25 @@ public:
     
     void addABLineData(double *a_values, double *b_values,
                        double *h_values, double *v_values, int data_count,
-                       std::vector<QColor> *color, std::vector<double> *lwd);
+                       std::vector<QColor> *color, std::vector<double> *alpha,
+                       std::vector<double> *lwd);
+    void addImageData(double *x_values, double *y_values, int data_count,
+                      QImage image, std::vector<double> *alpha);
     void addLineData(double *x_values, double *y_values, int data_count,
-                     std::vector<QColor> *color, std::vector<double> *lwd);
+                     std::vector<QColor> *color, std::vector<double> *alpha,
+                     std::vector<double> *lwd);
     void addPointData(double *x_values, double *y_values, int data_count,
                       std::vector<int> *symbol, std::vector<QColor> *color, std::vector<QColor> *border,
-                      std::vector<double> *lwd, std::vector<double> *size);
+                      std::vector<double> *alpha, std::vector<double> *lwd, std::vector<double> *size);
     void addTextData(double *x_values, double *y_values, std::vector<QString> *labels, int data_count,
-                     std::vector<QColor> *color, std::vector<double> *size, double *adj);
+                     std::vector<QColor> *color, std::vector<double> *alpha, std::vector<double> *size, double *adj);
     
     void addLegend(QtSLiM_LegendPosition position, int inset, double labelSize, double lineHeight,
                    double graphicsWidth, double exteriorMargin, double interiorMargin);
     void addLegendLineEntry(QString label, QColor color, double lwd);
     void addLegendPointEntry(QString label, int symbol, QColor color, QColor border, double lwd, double size);
     void addLegendSwatchEntry(QString label, QColor color);
+    void addLegendTitleEntry(QString label);
     
     virtual QString graphTitle(void) override;
     virtual QString aboutString(void) override;
@@ -109,10 +115,12 @@ private:
     std::vector<std::vector<int> *> symbol_;            // one symbol per point, OR one symbol for all points
     std::vector<std::vector<QColor> *> color_;          // one color per point, OR one color for all points
     std::vector<std::vector<QColor> *> border_;         // one border color per point, OR one for all points
+    std::vector<std::vector<double> *> alpha_;          // one alpha per point, OR one alpha for all points
     std::vector<std::vector<double> *> line_width_;     // one lwd per point, OR one lwd for all points
     std::vector<std::vector<double> *> size_;           // one size per point, OR one size for all points
     std::vector<double> xadj_;                          // one xadj for all points
     std::vector<double> yadj_;                          // one yadj for all points
+    std::vector<QImage> image_;                         // one QImage per image data; QImage() if unused
     
     void dataRange(std::vector<double *> &data, double *p_min, double *p_max);
     void rescaleAxesForDataRange(void);
@@ -122,6 +130,7 @@ private:
     void drawLines(QPainter &painter, QRect interiorRect, int dataIndex);
     void drawPoints(QPainter &painter, QRect interiorRect, int dataIndex);
     void drawText(QPainter &painter, QRect interiorRect, int dataIndex);
+    void drawImage(QPainter &painter, QRect interiorRect, int dataIndex);
     
     bool legend_added_ = false;                         // set to true by addLegend()
     QtSLiMLegendSpec legend_entries_;                   // unlike most graph types, we keep our legend around
