@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 2/28/2022.
-//  Copyright (c) 2022-2025 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2022-2025 Benjamin C. Haller.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -151,6 +151,14 @@ public:
 	bool warned_no_max_distance_ = false;
 	bool warned_readFromVCF_mutIDs_unused_ = false;
 	bool warned_no_ancestry_read_ = false;
+	bool warned_experiment_run_clocks_ = false;
+	bool warned_spatial_map_color_deprecated_ = false;
+	bool warned_spatial_map_image_deprecated_ = false;
+	
+	// checking for infinite loops, configurable with initializeSLiMOptions()
+#ifdef SLIMGUI
+	bool check_infinite_loops_ = true;
+#endif
 	
 	// these ivars are set around callbacks so we know what type of callback we're in, to prevent illegal operations during callbacks
 	// to make them easier to find, such checks should always be marked with a comment: // TIMING RESTRICTION
