@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 6/7/15.
-//  Copyright (c) 2015-2025 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2015-2025 Benjamin C. Haller.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -69,6 +69,10 @@ static inline bool SLiM_TokenIsCallbackIdentifier(EidosToken *token)
 		(token->token_string_.compare(gStr_reproduction) == 0))
 		return true;
 	
+	// also return true for obsolete callbacks, to parse it correctly even though it's obsolete
+	if (token->token_string_.compare(gStr_fitness) == 0)
+		return true;
+	
 	return false;
 }
 
