@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 1/4/15.
-//  Copyright (c) 2015-2025 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2015-2025 Benjamin C. Haller.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -44,8 +44,8 @@ class SLiMEidosBlock;
 
 
 // SLiM version: see also Info.plist and QtSLiM.pro
-#define SLIM_VERSION_STRING	("5.0")
-#define SLIM_VERSION_FLOAT	(5.0)
+#define SLIM_VERSION_STRING	("5.1")
+#define SLIM_VERSION_FLOAT	(5.1)
 
 
 // This should be called once at startup to give SLiM an opportunity to initialize static state
@@ -264,6 +264,15 @@ SLiMEidosBlock *SLiM_ExtractSLiMEidosBlockFromEidosValue_io(EidosValue *p_value,
 
 Species *SLiM_ExtractSpeciesFromEidosValue_No(EidosValue *p_value, int p_index, Community *p_community, const char *p_method_name);		// NULL tries for a single-species default
 
+// This template provides a function pointer that will delete a void* pointing to a given type.
+// It thus allows us to hold on to a void* pointing to a Qt object, for example, without building
+// against Qt; SLiMgui will give us the void* and a deleter pointer, which we can cache for it
+// and invalidate the cache, deleting the object, when appropriate without ever knowing its type.
+template<typename T>
+void Eidos_Deleter(void *ptr) {
+	delete static_cast<T*>(ptr);
+}
+
 
 // *******************************************************************************************************************
 //
@@ -799,6 +808,7 @@ extern const std::string &gStr_geneConversionGCBias;
 extern const std::string &gStr_geneConversionNonCrossoverFraction;
 extern const std::string &gStr_geneConversionMeanLength;
 extern const std::string &gStr_geneConversionSimpleConversionFraction;
+extern const std::string &gStr_chromosomeSubposition;
 extern const std::string &gStr_isNullHaplosome;
 extern const std::string &gStr_mutations;
 extern const std::string &gStr_uniqueMutations;
@@ -1110,10 +1120,13 @@ extern const std::string &gStr_plotWithTitle;
 extern const std::string &gStr_abline;
 extern const std::string &gStr_addLegend;
 extern const std::string &gStr_axis;
+extern const std::string &gStr_image;
 extern const std::string &gStr_legendLineEntry;
 extern const std::string &gStr_legendPointEntry;
 extern const std::string &gStr_legendSwatchEntry;
+extern const std::string &gStr_legendTitleEntry;
 extern const std::string &gStr_lines;
+extern const std::string &gStr_matrix;
 extern const std::string &gStr_points;
 extern const std::string &gStr_text;
 extern const std::string &gStr_title;
@@ -1252,6 +1265,7 @@ enum _SLiMGlobalStringID : int {
 	gID_geneConversionNonCrossoverFraction,
 	gID_geneConversionMeanLength,
 	gID_geneConversionSimpleConversionFraction,
+	gID_chromosomeSubposition,
 	gID_isNullHaplosome,
 	gID_mutations,
 	gID_uniqueMutations,
@@ -1563,10 +1577,13 @@ enum _SLiMGlobalStringID : int {
 	gID_abline,
 	gID_addLegend,
 	gID_axis,
+	gID_image,
 	gID_legendLineEntry,
 	gID_legendPointEntry,
 	gID_legendSwatchEntry,
+	gID_legendTitleEntry,
 	gID_lines,
+	gID_matrix,
 	gID_points,
 	gID_text,
 	gID_title,
