@@ -3,7 +3,7 @@
 //  Eidos
 //
 //  Created by Ben Haller on 10/8/20.
-//  Copyright (c) 2020-2025 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2020-2025 Benjamin C. Haller.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -96,6 +96,18 @@ EidosImage::EidosImage(int64_t p_width, int64_t p_height, bool p_grayscale) : wi
 
 EidosImage::~EidosImage(void)
 {
+#if defined(SLIMGUI)
+	if (image_)
+	{
+		if (image_deleter_)
+			image_deleter_(image_);
+		else
+			std::cout << "Missing Image image_deleter_; leaking memory" << std::endl;
+		
+		image_ = nullptr;
+		image_deleter_ = nullptr;
+	}
+#endif
 }
 
 const EidosClass *EidosImage::Class(void) const
