@@ -3,7 +3,7 @@
 //  Eidos
 //
 //  Created by Ben Haller on 10/12/20.
-//  Copyright (c) 2020-2025 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2020-2025 Benjamin C. Haller.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -618,8 +618,8 @@ EidosValue_SP EidosClass::ExecuteMethod_propertySignature(EidosGlobalStringID p_
 	
 	std::ostream &output_stream = p_interpreter.ExecutionOutputStream();
 	bool has_match_string = (p_arguments[0]->Type() == EidosValueType::kValueString);
-	EidosValue_String *propertyName_value = (EidosValue_String *)p_arguments[0].get();
-	const std::string &match_string = (has_match_string ? propertyName_value->StringRefAtIndex_NOCAST(0, nullptr) : gEidosStr_empty_string);
+	EidosValue *propertyName_value = (EidosValue *)p_arguments[0].get();
+	const std::string &match_string = (has_match_string ? ((EidosValue_String *)propertyName_value)->StringRefAtIndex_NOCAST(0, nullptr) : gEidosStr_empty_string);
 	const std::vector<EidosPropertySignature_CSP> *properties = Properties();
 	bool signature_found = false;
 	
@@ -650,10 +650,10 @@ EidosValue_SP EidosClass::ExecuteMethod_methodSignature(EidosGlobalStringID p_me
 	// before writing anything, erase a progress line if we've got one up, to try to make a clean slate
 	Eidos_EraseProgress();
 	
-	EidosValue_String *methodName_value = (EidosValue_String *)p_arguments[0].get();
+	EidosValue *methodName_value = p_arguments[0].get();
 	std::ostream &output_stream = p_interpreter.ExecutionOutputStream();
 	bool method_name_specified = (methodName_value->Type() == EidosValueType::kValueString);
-	const std::string &match_string = (method_name_specified ? methodName_value->StringRefAtIndex_NOCAST(0, nullptr) : gEidosStr_empty_string);
+	const std::string &match_string = (method_name_specified ? ((EidosValue_String *)methodName_value)->StringRefAtIndex_NOCAST(0, nullptr) : gEidosStr_empty_string);
 	const std::vector<EidosMethodSignature_CSP> *methods = Methods();
 	bool signature_found = false;
 	
