@@ -3,7 +3,7 @@
 //  Eidos
 //
 //  Created by Ben Haller on 4/6/15.
-//  Copyright (c) 2015-2025 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2015-2025 Benjamin C. Haller.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -54,6 +54,37 @@ R"V0G0N({
 		setwd(_oldwd);
 })V0G0N";
 
+// - (numeric)outerProduct(numeric x, numeric y)
+const char *gEidosSourceCode_outerProduct =
+R"V0G0N({
+	if (!isNULL(dim(x)) | !isNULL(dim(y)))
+		stop("ERROR (outerProduct): outerProduct() requires x and y to be vectors, not matrices or arrays; use drop() to convert to a vector if desired.");
+	return matrixMult(matrix(x), t(matrix(y)));
+})V0G0N";
+
+// - (numeric)matrixPow(numeric x, integer$ power)
+const char *gEidosSourceCode_matrixPow = 
+R"V0G0N({
+	d = dim(x);
+	
+	if (size(d) != 2)
+		stop("ERROR (matrixPow): matrixPow() requires x to be a matrix");
+	if (d[0] != d[1])
+		stop("ERROR (matrixPow): matrixPow() requires x to be a square matrix");
+	
+	if (power == 0)
+		return diag(d[0]);
+	else if (power == 1)
+		return x;
+	else if (power < 0) {
+		// check for singularity; we could just let inverse() raise an error, but it would be more confusing for the user
+		if (det(x) == 0)
+			stop("ERROR (matrixPow): in matrixPow() the vector x is singular and thus non-invertible, so it cannot be raised to a negative power.");
+		return matrixPow(inverse(x), -power);
+	} else // (power >= 2)
+		return matrixMult(x, matrixPow(x, power - 1));
+})V0G0N";
+
 
 //
 //	Construct our built-in function map
@@ -100,6 +131,7 @@ const std::vector<EidosFunctionSignature_CSP> &EidosInterpreter::BuiltInFunction
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("setIntersection",	Eidos_ExecuteFunction_setIntersection,	kEidosValueMaskAny))->AddAny("x")->AddAny("y"));
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("setDifference",		Eidos_ExecuteFunction_setDifference,	kEidosValueMaskAny))->AddAny("x")->AddAny("y"));
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("setSymmetricDifference",	Eidos_ExecuteFunction_setSymmetricDifference,	kEidosValueMaskAny))->AddAny("x")->AddAny("y"));
+		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("sign",				Eidos_ExecuteFunction_sign,			kEidosValueMaskNumeric))->AddNumeric("x"));
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("sin",				Eidos_ExecuteFunction_sin,			kEidosValueMaskFloat))->AddNumeric("x"));
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("sqrt",				Eidos_ExecuteFunction_sqrt,			kEidosValueMaskFloat))->AddNumeric("x"));
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("sum",				Eidos_ExecuteFunction_sum,			kEidosValueMaskNumeric | kEidosValueMaskSingleton))->AddLogicalEquiv("x"));
@@ -113,8 +145,9 @@ const std::vector<EidosFunctionSignature_CSP> &EidosInterpreter::BuiltInFunction
 		//	statistics functions
 		//
 		
-		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("cor",				Eidos_ExecuteFunction_cor,			kEidosValueMaskFloat | kEidosValueMaskSingleton))->AddNumeric("x")->AddNumeric("y"));
-		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("cov",				Eidos_ExecuteFunction_cov,			kEidosValueMaskFloat | kEidosValueMaskSingleton))->AddNumeric("x")->AddNumeric("y"));
+		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("cor",				Eidos_ExecuteFunction_cor,			kEidosValueMaskFloat))->AddNumeric("x")->AddNumeric_ON("y", gStaticEidosValueNULL));
+		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("cov",				Eidos_ExecuteFunction_cov,			kEidosValueMaskFloat))->AddNumeric("x")->AddNumeric_ON("y", gStaticEidosValueNULL));
+		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("filter",			Eidos_ExecuteFunction_filter,		kEidosValueMaskFloat))->AddNumeric("x")->AddFloat("filter")->AddLogicalEquiv_OS("outside", gStaticEidosValue_LogicalF));
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("max",				Eidos_ExecuteFunction_max,			kEidosValueMaskAnyBase | kEidosValueMaskSingleton))->AddAnyBase("x")->AddEllipsis());
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("mean",				Eidos_ExecuteFunction_mean,			kEidosValueMaskFloat | kEidosValueMaskSingleton))->AddLogicalEquiv("x"));
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("min",				Eidos_ExecuteFunction_min,			kEidosValueMaskAnyBase | kEidosValueMaskSingleton))->AddAnyBase("x")->AddEllipsis());
@@ -260,6 +293,10 @@ const std::vector<EidosFunctionSignature_CSP> &EidosInterpreter::BuiltInFunction
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("upperTri",			Eidos_ExecuteFunction_upperTri,		kEidosValueMaskLogical))->AddAny("x")->AddLogical_OS("diag", gStaticEidosValue_LogicalF));
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("lowerTri",			Eidos_ExecuteFunction_lowerTri,		kEidosValueMaskLogical))->AddAny("x")->AddLogical_OS("diag", gStaticEidosValue_LogicalF));
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("diag",				Eidos_ExecuteFunction_diag,			kEidosValueMaskAny))->AddAny_O("x", gStaticEidosValue_Integer1)->AddInt_OSN("nrow", gStaticEidosValueNULL)->AddInt_OSN("ncol", gStaticEidosValueNULL));
+		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("tr",				Eidos_ExecuteFunction_tr,			kEidosValueMaskNumeric | kEidosValueMaskSingleton))->AddNumeric("x"));
+		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("det",				Eidos_ExecuteFunction_det,			kEidosValueMaskNumeric | kEidosValueMaskSingleton))->AddNumeric("x"));
+		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("inverse",			Eidos_ExecuteFunction_inverse,		kEidosValueMaskFloat))->AddNumeric("x"));
+		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("asVector",			Eidos_ExecuteFunction_asVector,		kEidosValueMaskAny))->AddAny("x"));
 
 		
 		// ************************************************************************************
@@ -342,6 +379,10 @@ const std::vector<EidosFunctionSignature_CSP> &EidosInterpreter::BuiltInFunction
 		//
 		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature(gEidosStr_source,	gEidosSourceCode_source,	kEidosValueMaskVOID))->AddString_S(gEidosStr_filePath)->AddLogical_OS("chdir", gStaticEidosValue_LogicalF));
 		
+		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("matrixPow",			gEidosSourceCode_matrixPow,	kEidosValueMaskNumeric))->AddNumeric("x")->AddInt_S("power"));
+		
+		signatures->emplace_back((EidosFunctionSignature *)(new EidosFunctionSignature("outerProduct",		gEidosSourceCode_outerProduct,	kEidosValueMaskNumeric))->AddNumeric("x")->AddNumeric("y"));
+		
 		
 		// ************************************************************************************
 		//
@@ -852,10 +893,23 @@ EidosValue_SP UniqueEidosValue(const EidosValue *p_x_value, bool p_preserve_orde
 			std::vector<double> dup_vec(float_data, float_data + x_count);
 			
 			// sort NANs to the end
-			std::sort(dup_vec.begin(), dup_vec.end(), [](const double& a, const double& b) { return std::isnan(b) || (a < b); });
+			std::sort(dup_vec.begin(), dup_vec.end(), [](const double& a, const double& b) {
+				// If a is NaN and b is not NaN, a should come after b
+				if (std::isnan(a) && !std::isnan(b))
+					return false;
+				
+				// If b is NaN and a is not NaN, b should come after a
+				if (!std::isnan(a) && std::isnan(b))
+					return true;
+				
+				// If both are NaN or both are non-NaN, sort numerically (ascending)
+				return a < b;
+			});
 			
 			// Remove duplicates, including duplicate NANs
-			auto unique_iter = std::unique(dup_vec.begin(), dup_vec.end(), [](const double& a, const double& b) { return (std::isnan(a) && std::isnan(b)) || (a == b); });
+			auto unique_iter = std::unique(dup_vec.begin(), dup_vec.end(), [](const double& a, const double& b) {
+				return (std::isnan(a) && std::isnan(b)) || (a == b);
+			});
 			size_t unique_count = unique_iter - dup_vec.begin();
 			double *dup_ptr = dup_vec.data();
 			
