@@ -3,7 +3,7 @@
 //  Eidos
 //
 //  Created by Ben Haller on 7/11/20.
-//  Copyright (c) 2020-2025 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2020-2025 Benjamin C. Haller.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -21,6 +21,7 @@
 #include "eidos_test.h"
 
 #include <limits>
+#include <string>
 
 
 #pragma mark statistics
@@ -28,8 +29,8 @@ void _RunFunctionStatisticsTests_a_through_p(void)
 {
 	// cor()
 	EidosAssertScriptRaise("cor(T, T);", 0, "cannot be type");
-	EidosAssertScriptSuccess_NULL("cor(3, 3);");
-	EidosAssertScriptSuccess_NULL("cor(3.5, 3.5);");
+	EidosAssertScriptSuccess("cor(3, 3);", gStaticEidosValue_FloatNAN);
+	EidosAssertScriptSuccess("cor(3.5, 3.5);", gStaticEidosValue_FloatNAN);
 	EidosAssertScriptRaise("cor('foo', 'foo');", 0, "cannot be type");
 	EidosAssertScriptRaise("cor(c(F, F, T, F, T), c(F, F, T, F, T));", 0, "cannot be type");
 	EidosAssertScriptSuccess_L("abs(cor(1:5, 1:5) - 1) < 1e-10;", true);
@@ -48,14 +49,31 @@ void _RunFunctionStatisticsTests_a_through_p(void)
 	EidosAssertScriptRaise("cor(_Test(7), _Test(7));", 0, "cannot be type");
 	EidosAssertScriptRaise("cor(NULL, NULL);", 0, "cannot be type");
 	EidosAssertScriptRaise("cor(logical(0), logical(0));", 0, "cannot be type");
-	EidosAssertScriptSuccess_NULL("cor(integer(0), integer(0));");
-	EidosAssertScriptSuccess_NULL("cor(float(0), float(0));");
+	EidosAssertScriptSuccess("cor(integer(0), integer(0));", gStaticEidosValue_FloatNAN);
+	EidosAssertScriptSuccess("cor(float(0), float(0));", gStaticEidosValue_FloatNAN);
 	EidosAssertScriptRaise("cor(string(0), string(0));", 0, "cannot be type");
 	
+	std::string matrices = "a = c(1,2,3,4); b = c(1,2,4,5); c = c(1,3,4,5); d = c(5,3,2,1); m1 = cbind(a,b); m2 = cbind(c,d); m3 = cbind(a,b,c,d); ";
+	
+	EidosAssertScriptSuccess_L(matrices + "r = cor(m1); all(abs(r - matrix(c(1, 0.9899495, 0.9899495, 1), nrow=2)) < 1e-5);", true);
+	EidosAssertScriptSuccess_L(matrices + "r = cor(m2); all(abs(r - matrix(c(1, -1, -1, 1), nrow=2)) < 1e-5);", true);
+	EidosAssertScriptSuccess_L(matrices + "r = cor(m1, m2); all(abs(r - matrix(c(0.9827076, 0.9621405, -0.9827076, -0.9621405), nrow=2)) < 1e-5);", true);
+	EidosAssertScriptRaise(matrices + "r = cor(m1, t(m2));", 123, "incompatible dimensions");
+	EidosAssertScriptSuccess_L(matrices + "r = cor(m1, c); (nrow(r) == 2) & all(abs(r - matrix(c(0.9827076, 0.9621405), nrow=2)) < 1e-5);", true);
+	EidosAssertScriptSuccess_L(matrices + "r = cor(c, m1); (nrow(r) == 1) & all(abs(r - matrix(c(0.9827076, 0.9621405), ncol=2)) < 1e-5);", true);
+	EidosAssertScriptSuccess_L(matrices + "r = cor(m3); all(abs(r - matrix(c(1, 0.9899495, 0.9827076, -0.9827076, 0.9899495, 1, 0.9621405, -0.9621405, 0.9827076, 0.9621405, 1, -1, -0.9827076, -0.9621405, -1, 1), nrow=4)) < 1e-5);", true);
+	EidosAssertScriptSuccess_L("m4 = matrix(runif(36), nrow=6); r = cor(m4); identical(r, t(r));", true);
+	EidosAssertScriptSuccess_L("identical(cor(matrix(3), matrix(3)), matrix(NAN));", true);
+	EidosAssertScriptRaise("cor(matrix(3), matrix(c(3,5), ncol=1));", 0, "incompatible dimensions");
+	EidosAssertScriptSuccess_L("identical(cor(matrix(3), matrix(c(3,5), nrow=1)), matrix(c(NAN,NAN), nrow=1));", true);
+	EidosAssertScriptSuccess_L("identical(cor(matrix(c(3,5), nrow=1), matrix(3)), matrix(c(NAN,NAN), ncol=1));", true);
+	EidosAssertScriptSuccess_L("r = cor(matrix(c(3,5), ncol=1), matrix(c(3,5), ncol=1)); (nrow(r) == 1) & (ncol(r) == 1) & all(abs(r - matrix(1)) < 1e-14);", true);
+	EidosAssertScriptSuccess_L("identical(cor(matrix(c(3,5), nrow=1), matrix(c(3,5), nrow=1)), matrix(rep(NAN, 4), nrow=2));", true);
+	
 	// cov()
 	EidosAssertScriptRaise("cov(T, T);", 0, "cannot be type");
-	EidosAssertScriptSuccess_NULL("cov(3, 3);");
-	EidosAssertScriptSuccess_NULL("cov(3.5, 3.5);");
+	EidosAssertScriptSuccess("cov(3, 3);", gStaticEidosValue_FloatNAN);
+	EidosAssertScriptSuccess("cov(3.5, 3.5);", gStaticEidosValue_FloatNAN);
 	EidosAssertScriptRaise("cov('foo', 'foo');", 0, "cannot be type");
 	EidosAssertScriptRaise("cov(c(F, F, T, F, T), c(F, F, T, F, T));", 0, "cannot be type");
 	EidosAssertScriptSuccess_L("abs(cov(1:5, 1:5) - 2.5) < 1e-10;", true);
@@ -74,10 +92,59 @@ void _RunFunctionStatisticsTests_a_through_p(void)
 	EidosAssertScriptRaise("cov(_Test(7), _Test(7));", 0, "cannot be type");
 	EidosAssertScriptRaise("cov(NULL, NULL);", 0, "cannot be type");
 	EidosAssertScriptRaise("cov(logical(0), logical(0));", 0, "cannot be type");
-	EidosAssertScriptSuccess_NULL("cov(integer(0), integer(0));");
-	EidosAssertScriptSuccess_NULL("cov(float(0), float(0));");
+	EidosAssertScriptSuccess("cov(integer(0), integer(0));", gStaticEidosValue_FloatNAN);
+	EidosAssertScriptSuccess("cov(float(0), float(0));", gStaticEidosValue_FloatNAN);
 	EidosAssertScriptRaise("cov(string(0), string(0));", 0, "cannot be type");
 	
+	EidosAssertScriptSuccess_L(matrices + "r = cov(m1); all(abs(r - matrix(c(1.666667, 2.333333, 2.333333, 3.333333), nrow=2)) < 1e-5);", true);
+	EidosAssertScriptSuccess_L(matrices + "r = cov(m2); all(abs(r - matrix(c(2.916667, -2.916667, -2.916667, 2.916667), nrow=2)) < 1e-5);", true);
+	EidosAssertScriptSuccess_L(matrices + "r = cov(m1, m2); all(abs(r - matrix(c(2.166667, 3.000000, -2.166667, -3.000000), nrow=2)) < 1e-5);", true);
+	EidosAssertScriptRaise(matrices + "r = cov(m1, t(m2));", 123, "incompatible dimensions");
+	EidosAssertScriptSuccess_L(matrices + "r = cov(m1, c); (nrow(r) == 2) & all(abs(r - matrix(c(2.166667, 3.000000), nrow=2)) < 1e-5);", true);
+	EidosAssertScriptSuccess_L(matrices + "r = cov(c, m1); (nrow(r) == 1) & all(abs(r - matrix(c(2.166667, 3.000000), ncol=2)) < 1e-5);", true);
+	EidosAssertScriptSuccess_L(matrices + "r = cov(m3); all(abs(r - matrix(c(1.666667, 2.333333, 2.166667, -2.166667, 2.333333, 3.333333, 3.000000, -3.000000, 2.166667, 3.000000, 2.916667, -2.916667, -2.166667, -3.000000, -2.916667, 2.916667), nrow=4)) < 1e-5);", true);
+	EidosAssertScriptSuccess_L("m4 = matrix(runif(36), nrow=6); r = cov(m4); identical(r, t(r));", true);
+	EidosAssertScriptSuccess_L("identical(cov(matrix(3), matrix(3)), matrix(NAN));", true);
+	EidosAssertScriptRaise("cov(matrix(3), matrix(c(3,5), ncol=1));", 0, "incompatible dimensions");
+	EidosAssertScriptSuccess_L("identical(cov(matrix(3), matrix(c(3,5), nrow=1)), matrix(c(NAN,NAN), nrow=1));", true);
+	EidosAssertScriptSuccess_L("identical(cov(matrix(c(3,5), nrow=1), matrix(3)), matrix(c(NAN,NAN), ncol=1));", true);
+	EidosAssertScriptSuccess_L("r = cov(matrix(c(3,5), ncol=1), matrix(c(3,5), ncol=1)); (nrow(r) == 1) & (ncol(r) == 1) & all(abs(r - matrix(2.0)) < 1e-14);", true);
+	EidosAssertScriptSuccess_L("identical(cov(matrix(c(3,5), nrow=1), matrix(c(3,5), nrow=1)), matrix(rep(NAN, 4), nrow=2));", true);
+	
+	// filter()
+	EidosAssertScriptRaise("filter(1.0:10, float(0));", 0, "within the interval [1,");
+	EidosAssertScriptRaise("filter(1.0:10, 1.0:2);", 0, "length that is odd");
+	EidosAssertScriptSuccess_L("x = runif(100); identical(x, filter(x, 1.0));", true);
+	EidosAssertScriptSuccess_L("x = runif(100); identical(x * 2.0, filter(x, 2.0));", true);
+	EidosAssertScriptSuccess_L("x = runif(100); identical(x * -2.5, filter(x, -2.5));", true);
+	EidosAssertScriptSuccess_L("x = rep(NAN, 10); identical(x, filter(x, 1.0));", true);
+	EidosAssertScriptSuccess_FV("filter(1.0:10, rep(1/3, 3));", {std::numeric_limits<double>::quiet_NaN(), 2, 3, 4, 5, 6, 7, 8, 9, std::numeric_limits<double>::quiet_NaN()});
+	EidosAssertScriptSuccess_FV("filter(1.0:10, rep(1/5, 5));", {std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN(), 3, 4, 5, 6, 7, 8, std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN()});
+	EidosAssertScriptSuccess_FV("filter(1.0:10, rep(1.0, 5));", {std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN(), 15, 20, 25, 30, 35, 40, std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN()});
+	
+	EidosAssertScriptSuccess_L("r = filter(1.0:10, rep(1/5, 5), outside=T); sum(abs(r - c(2, 2.5, 3, 4, 5, 6, 7, 8, 8.5, 9))) < 1e-15;", true);
+	EidosAssertScriptSuccess_L("r = filter(1.0:10, rep(1/5, 5), outside=97); sum(abs(r - c(40, 21.4, 3, 4, 5, 6, 7, 8, 26.2, 44.2))) < 1e-12;", true);
+	EidosAssertScriptSuccess_L("r = filter(c(1.0, 2, 9, 10), rep(1/5, 5), outside=T); sum(abs(r - c(4, 5.5, 5.5, 7.0))) < 1e-15;", true);
+	EidosAssertScriptSuccess_L("r = filter(c(1.0, 2, 9, 10), rep(1/5, 5), outside=97); sum(abs(r - c(41.2, 23.8, 23.8, 43.0))) < 1e-12;", true);
+	EidosAssertScriptSuccess_L("r = filter(c(9.0), rep(1/5, 5), outside=T); sum(abs(r - c(9.0))) < 1e-15;", true);
+	EidosAssertScriptSuccess_L("r = filter(c(9.0), rep(1/5, 5), outside=97); sum(abs(r - c(79.4))) < 1e-12;", true);
+	
+	EidosAssertScriptRaise("filter(1:10, float(0));", 0, "within the interval [1,");
+	EidosAssertScriptRaise("filter(1:10, 1.0:2);", 0, "length that is odd");
+	EidosAssertScriptSuccess_L("x = rdunif(100, -100, 100); identical(x * 1.0, filter(x, 1.0));", true);
+	EidosAssertScriptSuccess_L("x = rdunif(100, -100, 100); identical(x * 2.0, filter(x, 2.0));", true);
+	EidosAssertScriptSuccess_L("x = rdunif(100); identical(x * -2.5, filter(x, -2.5));", true);
+	EidosAssertScriptSuccess_FV("filter(1:10, rep(1/3, 3));", {std::numeric_limits<double>::quiet_NaN(), 2, 3, 4, 5, 6, 7, 8, 9, std::numeric_limits<double>::quiet_NaN()});
+	EidosAssertScriptSuccess_FV("filter(1:10, rep(1/5, 5));", {std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN(), 3, 4, 5, 6, 7, 8, std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN()});
+	EidosAssertScriptSuccess_FV("filter(1:10, rep(1.0, 5));", {std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN(), 15, 20, 25, 30, 35, 40, std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN()});
+	
+	EidosAssertScriptSuccess_L("r = filter(1:10, rep(1/5, 5), outside=T); sum(abs(r - c(2, 2.5, 3, 4, 5, 6, 7, 8, 8.5, 9))) < 1e-15;", true);
+	EidosAssertScriptSuccess_L("r = filter(1:10, rep(1/5, 5), outside=97); sum(abs(r - c(40, 21.4, 3, 4, 5, 6, 7, 8, 26.2, 44.2))) < 1e-12;", true);
+	EidosAssertScriptSuccess_L("r = filter(c(1, 2, 9, 10), rep(1/5, 5), outside=T); sum(abs(r - c(4, 5.5, 5.5, 7.0))) < 1e-15;", true);
+	EidosAssertScriptSuccess_L("r = filter(c(1, 2, 9, 10), rep(1/5, 5), outside=97); sum(abs(r - c(41.2, 23.8, 23.8, 43.0))) < 1e-12;", true);
+	EidosAssertScriptSuccess_L("r = filter(c(9), rep(1/5, 5), outside=T); sum(abs(r - c(9.0))) < 1e-15;", true);
+	EidosAssertScriptSuccess_L("r = filter(c(9), rep(1/5, 5), outside=97); sum(abs(r - c(79.4))) < 1e-12;", true);
+	
 	// max()
 	EidosAssertScriptSuccess_L("max(T);", true);
 	EidosAssertScriptSuccess_I("max(3);", 3);
@@ -438,8 +505,8 @@ void _RunFunctionStatisticsTests_q_through_z(void)
 	
 	// sd()
 	EidosAssertScriptRaise("sd(T);", 0, "cannot be type");
-	EidosAssertScriptSuccess_NULL("sd(3);");
-	EidosAssertScriptSuccess_NULL("sd(3.5);");
+	EidosAssertScriptSuccess("sd(3);", gStaticEidosValue_FloatNAN);
+	EidosAssertScriptSuccess("sd(3.5);", gStaticEidosValue_FloatNAN);
 	EidosAssertScriptRaise("sd('foo');", 0, "cannot be type");
 	EidosAssertScriptRaise("sd(c(F, F, T, F, T));", 0, "cannot be type");
 	EidosAssertScriptSuccess_F("sd(c(2, 3, 2, 8, 0));", 3);
@@ -449,8 +516,8 @@ void _RunFunctionStatisticsTests_q_through_z(void)
 	EidosAssertScriptRaise("sd(_Test(7));", 0, "cannot be type");
 	EidosAssertScriptRaise("sd(NULL);", 0, "cannot be type");
 	EidosAssertScriptRaise("sd(logical(0));", 0, "cannot be type");
-	EidosAssertScriptSuccess_NULL("sd(integer(0));");
-	EidosAssertScriptSuccess_NULL("sd(float(0));");
+	EidosAssertScriptSuccess("sd(integer(0));", gStaticEidosValue_FloatNAN);
+	EidosAssertScriptSuccess("sd(float(0));", gStaticEidosValue_FloatNAN);
 	EidosAssertScriptRaise("sd(string(0));", 0, "cannot be type");
 	
 	// ttest()
@@ -471,8 +538,8 @@ void _RunFunctionStatisticsTests_q_through_z(void)
 	
 	// var()
 	EidosAssertScriptRaise("var(T);", 0, "cannot be type");
-	EidosAssertScriptSuccess_NULL("var(3);");
-	EidosAssertScriptSuccess_NULL("var(3.5);");
+	EidosAssertScriptSuccess("var(3);", gStaticEidosValue_FloatNAN);
+	EidosAssertScriptSuccess("var(3.5);", gStaticEidosValue_FloatNAN);
 	EidosAssertScriptRaise("var('foo');", 0, "cannot be type");
 	EidosAssertScriptRaise("var(c(F, F, T, F, T));", 0, "cannot be type");
 	EidosAssertScriptSuccess_F("var(c(2, 3, 2, 8, 0));", 9);
@@ -482,8 +549,8 @@ void _RunFunctionStatisticsTests_q_through_z(void)
 	EidosAssertScriptRaise("var(_Test(7));", 0, "cannot be type");
 	EidosAssertScriptRaise("var(NULL);", 0, "cannot be type");
 	EidosAssertScriptRaise("var(logical(0));", 0, "cannot be type");
-	EidosAssertScriptSuccess_NULL("var(integer(0));");
-	EidosAssertScriptSuccess_NULL("var(float(0));");
+	EidosAssertScriptSuccess("var(integer(0));", gStaticEidosValue_FloatNAN);
+	EidosAssertScriptSuccess("var(float(0));", gStaticEidosValue_FloatNAN);
 	EidosAssertScriptRaise("var(string(0));", 0, "cannot be type");
 }
 
