@@ -9,17 +9,10 @@
 #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
 #pragma clang diagnostic ignored "-Wshorten-64-to-32"
 
-/* Local functions */
-local int gz_init OF((gz_statep));
-local int gz_comp OF((gz_statep, int));
-local int gz_zero OF((gz_statep, z_off64_t));
-local z_size_t gz_write OF((gz_statep, voidpc, z_size_t));
-
 /* Initialize state for writing a gzip file.  Mark initialization by setting
    state->size to non-zero.  Return -1 on a memory allocation failure, or 0 on
    success. */
-local int gz_init(gz_statep state)	// BCH: rearranged to get rid of prototype warning
-{
+local int gz_init(gz_statep state) {
     int ret;
     z_streamp strm = &(state->strm);
 
@@ -73,8 +66,7 @@ local int gz_init(gz_statep state)	// BCH: rearranged to get rid of prototype wa
    deflate() flush value.  If flush is Z_FINISH, then the deflate() state is
    reset to start a new gzip stream.  If gz->direct is true, then simply write
    to the output file without compressing, and ignore flush. */
-local int gz_comp(gz_statep state, int flush)	// BCH: rearranged to get rid of prototype warning
-{
+local int gz_comp(gz_statep state, int flush) {
     int ret, writ;
     unsigned have, put, max = ((unsigned)-1 >> 2) + 1;
     z_streamp strm = &(state->strm);
@@ -152,8 +144,7 @@ local int gz_comp(gz_statep state, int flush)	// BCH: rearranged to get rid of p
 
 /* Compress len zeros to output.  Return -1 on a write error or memory
    allocation failure by gz_comp(), or 0 on success. */
-local int gz_zero(gz_statep state, z_off64_t len)	// BCH: rearranged to get rid of prototype warning
-{
+local int gz_zero(gz_statep state, z_off64_t len) {
     int first;
     unsigned n;
     z_streamp strm = &(state->strm);
@@ -183,8 +174,7 @@ local int gz_zero(gz_statep state, z_off64_t len)	// BCH: rearranged to get rid
 
 /* Write len bytes from buf to file.  Return the number of bytes written.  If
    the returned value is less than len, then there was an error. */
-local z_size_t gz_write(gz_statep state, voidpc buf, z_size_t len)	// BCH: rearranged to get rid of prototype warning
-{
+local z_size_t gz_write(gz_statep state, voidpc buf, z_size_t len) {
     z_size_t put = len;
 
     /* if len is zero, avoid unnecessary operations */
@@ -248,8 +238,7 @@ local z_size_t gz_write(gz_statep state, voidpc buf, z_size_t len)	// BCH: rearr
 }
 
 /* -- see zlib.h -- */
-int ZEXPORT gzwrite(gzFile file, voidpc buf, unsigned len)	// BCH: rearranged to get rid of prototype warning
-{
+int ZEXPORT gzwrite(gzFile file, voidpc buf, unsigned len) {
     gz_statep state;
 
     /* get internal structure */
@@ -273,8 +262,8 @@ int ZEXPORT gzwrite(gzFile file, voidpc buf, unsigned len)	// BCH: rearranged to
 }
 
 /* -- see zlib.h -- */
-z_size_t ZEXPORT gzfwrite(voidpc buf, z_size_t size, z_size_t nitems, gzFile file)	// BCH: rearranged to get rid of prototype warning
-{
+z_size_t ZEXPORT gzfwrite(voidpc buf, z_size_t size, z_size_t nitems,
+                          gzFile file) {
     z_size_t len;
     gz_statep state;
 
@@ -299,8 +288,7 @@ z_size_t ZEXPORT gzfwrite(voidpc buf, z_size_t size, z_size_t nitems, gzFile fil
 }
 
 /* -- see zlib.h -- */
-int ZEXPORT gzputc(gzFile file, int c)	// BCH: rearranged to get rid of prototype warning
-{
+int ZEXPORT gzputc(gzFile file, int c) {
     unsigned have;
     unsigned char buf[1];
     gz_statep state;
@@ -345,8 +333,7 @@ int ZEXPORT gzputc(gzFile file, int c)	// BCH: rearranged to get rid of prototyp
 }
 
 /* -- see zlib.h -- */
-int ZEXPORT gzputs(gzFile file, const char *s)	// BCH: rearranged to get rid of prototype warning
-{
+int ZEXPORT gzputs(gzFile file, const char *s) {
     z_size_t len, put;
     gz_statep state;
 
@@ -373,8 +360,7 @@ int ZEXPORT gzputs(gzFile file, const char *s)	// BCH: rearranged to get rid of
 #include <stdarg.h>
 
 /* -- see zlib.h -- */
-int ZEXPORTVA gzvprintf(gzFile file, const char *format, va_list va)
-{
+int ZEXPORTVA gzvprintf(gzFile file, const char *format, va_list va) {
     int len;
     unsigned left;
     char *next;
@@ -445,8 +431,7 @@ int ZEXPORTVA gzvprintf(gzFile file, const char *format, va_list va)
     return len;
 }
 
-int ZEXPORTVA gzprintf(gzFile file, const char *format, ...)
-{
+int ZEXPORTVA gzprintf(gzFile file, const char *format, ...) {
     va_list va;
     int ret;
 
@@ -459,13 +444,10 @@ int ZEXPORTVA gzprintf(gzFile file, const char *format, ...)
 #else /* !STDC && !Z_HAVE_STDARG_H */
 
 /* -- see zlib.h -- */
-int ZEXPORTVA gzprintf(file, format, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10,
-                       a11, a12, a13, a14, a15, a16, a17, a18, a19, a20)
-    gzFile file;
-    const char *format;
-    int a1, a2, a3, a4, a5, a6, a7, a8, a9, a10,
-        a11, a12, a13, a14, a15, a16, a17, a18, a19, a20;
-{
+int ZEXPORTVA gzprintf(gzFile file, const char *format, int a1, int a2, int a3,
+                       int a4, int a5, int a6, int a7, int a8, int a9, int a10,
+                       int a11, int a12, int a13, int a14, int a15, int a16,
+                       int a17, int a18, int a19, int a20) {
     unsigned len, left;
     char *next;
     gz_statep state;
@@ -547,8 +529,7 @@ int ZEXPORTVA gzprintf(file, format, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10,
 #endif
 
 /* -- see zlib.h -- */
-int ZEXPORT gzflush(gzFile file, int flush)	// BCH: rearranged to get rid of prototype warning
-{
+int ZEXPORT gzflush(gzFile file, int flush) {
     gz_statep state;
 
     /* get internal structure */
@@ -577,8 +558,7 @@ int ZEXPORT gzflush(gzFile file, int flush)	// BCH: rearranged to get rid of pro
 }
 
 /* -- see zlib.h -- */
-int ZEXPORT gzsetparams(gzFile file, int level, int strategy)	// BCH: rearranged to get rid of prototype warning
-{
+int ZEXPORT gzsetparams(gzFile file, int level, int strategy) {
     gz_statep state;
     z_streamp strm;
 
@@ -589,7 +569,7 @@ int ZEXPORT gzsetparams(gzFile file, int level, int strategy)	// BCH: rearranged
     strm = &(state->strm);
 
     /* check that we're writing and that there's no error */
-    if (state->mode != GZ_WRITE || state->err != Z_OK)
+    if (state->mode != GZ_WRITE || state->err != Z_OK || state->direct)
         return Z_STREAM_ERROR;
 
     /* if no change is requested, then do nothing */
@@ -616,8 +596,7 @@ int ZEXPORT gzsetparams(gzFile file, int level, int strategy)	// BCH: rearranged
 }
 
 /* -- see zlib.h -- */
-int ZEXPORT gzclose_w(gzFile file)	// BCH: rearranged to get rid of prototype warning
-{
+int ZEXPORT gzclose_w(gzFile file) {
     int ret = Z_OK;
     gz_statep state;
 
